;;; -*- Common-Lisp -*-

;;;  ffi.lisp --- binge delcarations

;;;  Copyright (C) 2004 by Walter C. Pelissero

;;;  Author: Walter C. Pelissero <walter@pelissero.de>
;;;  Project: Common Lisp Terminal I/O
;;;  $Id: F-49EE1AF0B7829651C12258F80C1636C0.lisp,v 1.1.1.1 2005/01/18 22:29:11 wcp Exp $

;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License as
;;; published by the Free Software Foundation; either version 2, or (at
;;; your option) any later version.
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; General Public License for more details.
;;; You should have received a copy of the GNU General Public License
;;; along with this program; see the file COPYING.  If not, write to
;;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;;; Boston, MA 02111-1307, USA.


#+cmu (ext:file-comment "$Id: F-49EE1AF0B7829651C12258F80C1636C0.lisp,v 1.1.1.1 2005/01/18 22:29:11 wcp Exp $")

(in-package :cl-user)

(defpackage :cltio-binge
  (:use :common-lisp :binge)
  (:export #:create-ffi))

(in-package :cltio-binge)

(defvar *ffi-includes* '("errno.h" "termios.h"))

(defvar *ffi-constants*
  '(("TCSANOW" :doc "The change occurs immediately.")
    ("TCSADRAIN"
     :doc "The change occurs after all output written to fd has been transmitted to the terminal.  This value of action should be used when changing parameters that affect output.")
    ("TCSAFLUSH"
     :doc "The change occurs after all output written to fd has been transmitted to the terminal.  Additionally, any input that has been received but not read is discarded.")
    ("TCSASOFT"
     :doc "If this value is or'ed into the action value, the values of the c_cflag, c_ispeed, and c_ospeed fields are ignored.")
    ;; input modes
    ("IGNBRK" :doc "ignore BREAK condition")
    ("BRKINT" :doc "map BREAK to SIGINTR")
    ("IGNPAR" :doc "ignore (discard) parity errors")
    ("PARMRK" :doc "mark parity and framing errors")
    ("INPCK" :doc "enable checking of parity errors")
    ("ISTRIP" :doc "strip 8th bit off chars")
    ("INLCR" :doc "map NL into CR")
    ("IGNCR" :doc "ignore CR")
    ("ICRNL" :doc "map CR to NL (ala CRMOD)")
    ("IXON" :doc "enable output flow control")
    ("IXOFF" :doc "enable input flow control")
    ("IXANY" :doc "any char will restart after stop")
    ("IMAXBEL" :doc "ring bell on input queue full")
    ;; output modes
    ("OPOST" :doc "enable following output processing")
    ("ONLCR" :doc "map NL to CR-NL (ala CRMOD)")
    ("OCRNL" :doc "map CR to NL")
    ("OXTABS" :doc "expand tabs to spaces")
    ("ONOEOT" :doc "discard EOT's `^D' on output)")
    ("ONOCR" :doc "do not transmit CRs on column 0")
    ("ONLRET" :doc "on the terminal NL performs the CR function")
    ;; control modes
    ("CSIZE" :doc "character size mask")
    ("CS5" :doc "5 bits (pseudo)")
    ("CS6" :doc "6 bits")
    ("CS7" :doc "7 bits")
    ("CS8" :doc "8 bits")
    ("CSTOPB" :doc "send 2 stop bits")
    ("CREAD" :doc "enable receiver")
    ("PARENB" :doc "parity enable")
    ("PARODD" :doc "odd parity, else even")
    ("HUPCL" :doc "hang up on last close")
    ("CLOCAL" :doc "ignore modem status lines")
    ("CCTS_OFLOW" :doc "CTS flow control of output")
    ("CRTSCTS" :doc "same as CCTS_OFLOW")
    ("CRTS_IFLOW" :doc "RTS flow control of input")
    ("MDMBUF" :doc "flow control output via Carrier")
    ;; local modes
    ("ECHOKE" :doc "visual erase for line kill")
    ("ECHOE" :doc "visually erase chars")
    ("ECHO" :doc "enable echoing")
    ("ECHONL" :doc "echo NL even if ECHO is off")
    ("ECHOPRT" :doc "visual erase mode for hardcopy")
    ("ECHOCTL" :doc "echo control chars as ^(Char)")
    ("ISIG" :doc "enable signals INTR, QUIT, [D]SUSP")
    ("ICANON" :doc "canonicalize input lines")
    ("ALTWERASE" :doc "use alternate WERASE algorithm")
    ("IEXTEN" :doc "enable DISCARD and LNEXT")
    ("EXTPROC" :doc "external processing")
    ("TOSTOP" :doc "stop background jobs from output")
    ("FLUSHO" :doc "output being flushed (state)")
    ("NOKERNINFO" :doc "no kernel output from VSTATUS")
    ("PENDIN" :doc "XXX retype pending input (state)")
    ("NOFLSH" :doc "don't flush after interrupt")))

(defvar *ffi-functions*
  '(("tcgetattr" :doc "Get the termios structure.")
    ("tcsetattr" :doc "Set the termios structure.")
    ("cfgetispeed" :doc "Get terminal input speed from termios structure.")
    ("cfsetispeed" :doc "Set terminal input speed in termios structure.")
    ("cfgetospeed" :doc "Get terminal output speed from termios structure.")
    ("cfsetospeed" :doc "Set terminal output speed in termios structure.")
    ("cfmakeraw" :doc "Set the termios structure so that all I/O pre/post-processing is disabled.")
    ("cfsetspeed" :doc "Set terminal input and output speed in termios structure.")
    ;; modern UNIX don't have the global variable errno (for thread
    ;; safety?)
    ("__error" :doc "Return the last system error code.")))

(defvar *ffi-variables* '(("errno" :doc "Last system error code.")))

(defvar *ffi-types* '())

(defun create-ffi ()
  (let ((namespace (make-instance 'c-namespace))
	;; generate a portable FFI
	(binge:*backend* (make-instance 'binge-backend:uffi-backend)))
    (binge-compiler:add-declaration *gcc-builtin-va_list* namespace)
    (populate-namespace namespace
			(extract-from-headers *ffi-includes*
					      :defines *gcc-null-defines*
					      :constants *ffi-constants*))
    (make-ffi #P"cltio.lisp" namespace
	      :variables *ffi-variables*
	      :functions *ffi-functions*
	      :constants *ffi-constants*
	      :types *ffi-types*)))
